{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2019                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Crypto;

interface

uses
  Classes, JS, Web, Types;

type
  TCryptoEncSignEvent = procedure(ADataAsArrayBuffer: TJSArrayBuffer) of object;
  TCryptoVerifyEvent = procedure(AValid: Boolean) of object;
  TCryptoDecryptStringEvent = procedure(AData: string) of object;
  TCryptoDecryptBinaryEvent = procedure(AData: TJSUint8Array) of object;

  TCryptoKeyUsage = (kuEncrypt, kuDecrypt, kuSign, kuVerify, kuDeriveKey, kuDeriveBits, kuWrapKey, kuUnwrapKey);
  TCryptoKeyUsages = set of TCryptoKeyUsage;

  TAESEncryptionType = (aetCBC, aetGCM);
  TAESEncryptionKeyLength = (akl128, akl192, akl256);
  TCryptoDecryptResultType = (drtString, drtBinary);
  TCryptoExportImportFormat = (efRaw, efPKCS8, efSPKI, efJSON);

  TCryptoRawExportWrapEvent = procedure(ABinary: TJSUint8Array) of object;
  TCryptoJSONExportWrapEvent = procedure(AJSON: string) of object;
  TCryptoStringExportWrapEvent = procedure(AValue: string; ABinary: TJSUint8Array) of object;
  TCryptoKeyUnwrappedEvent = procedure(AUnwrappedKey: TJSCryptoKey) of object;
  TCryptoKeyWrappedEvent = procedure(AWrappedKey: TJSArrayBuffer) of object;
  TCryptoErrorEvent = procedure(AError: string) of object;

  TCryptoBase = class(TPersistent)
  private
    FOnKeyExportedSPKI: TCryptoStringExportWrapEvent;
    FExtractableKey: Boolean;
    FOnKeyExportedRaw: TCryptoRawExportWrapEvent;
    FOnKeyExportedJSON: TCryptoJSONExportWrapEvent;
    FUsages: TCryptoKeyUsages;
    FOnKeyCreated: TNotifyEvent;
    FOnKeyExportedPKCS: TCryptoStringExportWrapEvent;
    FOnError: TCryptoErrorEvent;
  protected
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedPKCS8: TCryptoStringExportWrapEvent read FOnKeyExportedPKCS write FOnKeyExportedPKCS;
    property OnKeyExportedRaw: TCryptoRawExportWrapEvent read FOnKeyExportedRaw write FOnKeyExportedRaw;
    property OnKeyExportedSPKI: TCryptoStringExportWrapEvent read FOnKeyExportedSPKI write FOnKeyExportedSPKI;
  public
    property ExtractableKey: Boolean read FExtractableKey write FExtractableKey default False;
    property Usages: TCryptoKeyUsages read FUsages write FUsages default [kuEncrypt, kuDecrypt];
    property OnKeyCreated: TNotifyEvent read FOnKeyCreated write FOnKeyCreated;
    property OnError: TCryptoErrorEvent read FOnError write FOnError;
  end;

  TEncryptionBase = class(TCryptoBase)
  private
    FOnDecryptedString: TCryptoDecryptStringEvent;
    FOnDecryptedBinary: TCryptoDecryptBinaryEvent;
    FOnKeyWrappedEvent: TCryptoKeyWrappedEvent;
    FOnKeyUnwrapped: TCryptoKeyUnwrappedEvent;
    FOnEncrypted: TCryptoEncSignEvent;
  public
    procedure Encrypt(APlainText: string); overload; virtual;
    procedure Encrypt(ABinary: TJSUint8Array); overload; virtual;
    procedure Decrypt(AEncryptedData: TJSArrayBuffer; AResultType: TCryptoDecryptResultType); virtual;
    property OnDecryptedBinary: TCryptoDecryptBinaryEvent read FOnDecryptedBinary write FOnDecryptedBinary;
    property OnDecryptedString: TCryptoDecryptStringEvent read FOnDecryptedString write FOnDecryptedString;
    property OnEncrypted: TCryptoEncSignEvent read FOnEncrypted write FOnEncrypted;
    property OnKeyUnwrapped: TCryptoKeyUnwrappedEvent read FOnKeyUnwrapped write FOnKeyUnwrapped;
    property OnKeyWrapped: TCryptoKeyWrappedEvent read FOnKeyWrappedEvent write FOnKeyWrappedEvent;
  end;

  TWebAESEncryption = class(TEncryptionBase)
  private
    FKeyLength: TAESEncryptionKeyLength;
    FType: TAESEncryptionType;
    FCryptoKey: TJSCryptoKey;
    FOnKeyImported: TNotifyEvent;
  public
    constructor Create(AType: TAESEncryptionType); overload;
    constructor Create(AType: TAESEncryptionType; AKeyLength: TAESEncryptionKeyLength); overload;
    constructor Create(AType: TAESEncryptionType; AKeyLength: TAESEncryptionKeyLength; AExtractable: Boolean); overload;
    constructor Create(AType: TAESEncryptionType; AKeyLength: TAESEncryptionKeyLength; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure Encrypt(APlainText: string); overload; override;
    procedure Encrypt(ABinary: TJSUint8Array); overload; override;
    procedure ExportKey(AFormat: TCryptoExportImportFormat);
    procedure Decrypt(AEncryptedData: TJSArrayBuffer; AResultType: TCryptoDecryptResultType); override;
    procedure GenerateKey;
    procedure ImportKey(AJSON: string); overload;
    procedure ImportKey(ABinary: TJSUint8Array); overload;
    procedure ImportKey(ARaw: TJSArrayBuffer); overload;
    procedure UnwrapKey(AImportFormat: TCryptoExportImportFormat; AKey: TJSArrayBuffer; AKeyAlgorithm: JSValue; AExtractable: Boolean; AKeyUsages: TCryptoKeyUsages); overload;
    procedure WrapKey(AKey: TJSCryptoKey; AExportFormat: TCryptoExportImportFormat);
    property AESType: TAESEncryptionType read FType write FType default aetCBC;
    property CryptoKey: TJSCryptoKey read FCryptoKey write FCryptoKey;
    property KeyLength: TAESEncryptionKeyLength read FKeyLength write FKeyLength default akl128;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedRaw: TCryptoRawExportWrapEvent read FOnKeyExportedRaw write FOnKeyExportedRaw;
    property OnKeyImported: TNotifyEvent read FOnKeyImported write FOnKeyImported;
  end;

  TRSAModulusLength = (rml2048, rml3072, rml4096);
  TCryptoHash = (ehSHA1, ehSHA256, ehSHA384, ehSHA512);
  TCryptoAsymKeyType = (aktPrivate, aktPublic);

  TWebRSAEncryption = class(TEncryptionBase)
  private
    FPrivateKey: TJSCryptoKey;
    FHash: TCryptoHash;
    FOnPublicKeyImported: TNotifyEvent;
    FPublicKey: TJSCryptoKey;
    FModulusLength: TRSAModulusLength;
    FOnPrivateKeyImported: TNotifyEvent;
  public
    constructor Create; overload;
    constructor Create(AModulusLength: TRSAModulusLength); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure Encrypt(APlainText: string); overload; override;
    procedure Encrypt(ABinary: TJSUint8Array); overload; override;
    procedure ExportKey(AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat);
    procedure Decrypt(AEncryptedData: TJSArrayBuffer; AResultType: TCryptoDecryptResultType); override;
    procedure GenerateKey;
    procedure ImportKey(AKey: string; AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat); overload;
    procedure ImportKey(ABinary: TJSUint8Array; AKeyType: TCryptoAsymKeyType); overload; //PKCS#8 if private, SPKI if public
    procedure WrapKey(AKey: TJSCryptoKey; AExportFormat: TCryptoExportImportFormat);
    procedure UnwrapKey(AImportFormat: TCryptoExportImportFormat; AKey: TJSArrayBuffer; AKeyAlgorithm: JSValue; AExtractable: Boolean; AKeyUsages: TCryptoKeyUsages);
    property Hash: TCryptoHash read FHash write FHash;
    property ModulusLength: TRSAModulusLength read FModulusLength write FModulusLength;
    property PrivateKey: TJSCryptoKey read FPrivateKey write FPrivateKey;
    property PublicKey: TJSCryptoKey read FPublicKey write FPublicKey;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedPKCS8: TCryptoStringExportWrapEvent read FOnKeyExportedPKCS write FOnKeyExportedPKCS;
    property OnKeyExportedSPKI: TCryptoStringExportWrapEvent read FOnKeyExportedSPKI write FOnKeyExportedSPKI;
    property OnPrivateKeyImported: TNotifyEvent read FOnPrivateKeyImported write FOnPrivateKeyImported;
    property OnPublicKeyImported: TNotifyEvent read FOnPublicKeyImported write FOnPublicKeyImported;
  end;

  TSignatureBase = class(TCryptoBase)
  private
    FOnSigned: TCryptoEncSignEvent;
    FOnVerify: TCryptoVerifyEvent;
  public
    procedure Sign(AText: string); overload; virtual;
    procedure Sign(ABinary: TJSUint8Array); overload; virtual;
    procedure Verify(ASignature: TJSArrayBuffer; AData: TJSArrayBuffer); virtual;
    property OnSigned: TCryptoEncSignEvent read FOnSigned write FOnSigned;
    property OnVerify: TCryptoVerifyEvent read FOnVerify write FOnVerify;
  end;

  TWebRSASignature = class(TSignatureBase)
  private
    FPrivateKey: TJSCryptoKey;
    FHash: TCryptoHash;
    FOnPublicKeyImported: TNotifyEvent;
    FPublicKey: TJSCryptoKey;
    FModulusLength: TRSAModulusLength;
    FOnPrivateKeyImported: TNotifyEvent;
  public
    constructor Create; overload;
    constructor Create(AModulusLength: TRSAModulusLength); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure ExportKey(AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat);
    procedure GenerateKey;
    procedure ImportKey(AKey: string; AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat); overload;
    procedure ImportKey(ABinary: TJSUint8Array; AKeyType: TCryptoAsymKeyType); overload; //PKCS#8 if private, SPKI if public
    procedure Sign(AText: string); overload; override;
    procedure Sign(ABinary: TJSUint8Array); overload; override;
    procedure Verify(ASignature: TJSArrayBuffer; AData: TJSArrayBuffer); override;
    property Hash: TCryptoHash read FHash;
    property ModulusLength: TRSAModulusLength read FModulusLength;
    property PrivateKey: TJSCryptoKey read FPrivateKey write FPrivateKey;
    property PublicKey: TJSCryptoKey read FPublicKey write FPublicKey;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedPKCS8: TCryptoStringExportWrapEvent read FOnKeyExportedPKCS write FOnKeyExportedPKCS;
    property OnKeyExportedSPKI: TCryptoStringExportWrapEvent read FOnKeyExportedSPKI write FOnKeyExportedSPKI;
    property OnPrivateKeyImported: TNotifyEvent read FOnPrivateKeyImported write FOnPrivateKeyImported;
    property OnPublicKeyImported: TNotifyEvent read FOnPublicKeyImported write FOnPublicKeyImported;
  end;

  TWebHMACSignature = class(TSignatureBase)
  private
    FHash: TCryptoHash;
    FCryptoKey: TJSCryptoKey;
    FOnKeyImported: TNotifyEvent;
  public
    constructor Create; overload;
    constructor Create(AHash: TCryptoHash); overload;
    constructor Create(AHash: TCryptoHash; AExtractable: Boolean); overload;
    constructor Create(AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure GenerateKey;
    procedure ExportKey(AFormat: TCryptoExportImportFormat);
    procedure ImportKey(AJSON: string); overload;
    procedure ImportKey(ABinary: TJSUint8Array); overload;
    procedure ImportKey(ARaw: TJSArrayBuffer); overload;
    procedure Sign(AText: string); overload; override;
    procedure Sign(ABinary: TJSUint8Array); overload; override;
    procedure Verify(ASignature: TJSArrayBuffer; AData: TJSArrayBuffer); override;
    property CryptoKey: TJSCryptoKey read FCryptoKey write FCryptoKey;
    property Hash: TCryptoHash read FHash;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedRaw: TCryptoRawExportWrapEvent read FOnKeyExportedRaw write FOnKeyExportedRaw;
    property OnKeyImported: TNotifyEvent read FOnKeyImported write FOnKeyImported;
  end;

var
  TCryptoKeyUsageValue: array[TCryptoKeyUsage] of string = ('encrypt', 'decrypt', 'sign', 'verify', 'deriveKey',
    'deriveBits', 'wrapKey', 'unwrapKey');

const
  IVLEN = 16;

function ABToBase(AArrayBuffer: TJSArrayBuffer): string;
function ABToBinaryString(AArrayBuffer: TJSArrayBuffer): string;
function BaseToAB(ABase: string): TJSArrayBuffer;
function BinaryStringToAB(ABinaryString: string): TJSArrayBuffer;
function GetMessageEncoding(APlainText: string): TJSUint8Array;
function GetMessageStringDecoding(AArray: TJSArrayBuffer): string;
function IsTextEncoderSupported: Boolean;

implementation

function ABToBase(AArrayBuffer: TJSArrayBuffer): string;
begin
  Result := '';
end;

function ABToBinaryString(AArrayBuffer: TJSArrayBuffer): string;
begin
  Result := '';
end;

function BaseToAB(ABase: string): TJSArrayBuffer;
begin
  Result := nil;
end;

function BinaryStringToAB(ABinaryString: string): TJSArrayBuffer;
begin
  Result := nil;
end;

function GetMessageEncoding(APlainText: string): TJSUint8Array;
begin
  Result := nil;
end;

function GetMessageStringDecoding(AArray: TJSArrayBuffer): string;
begin
  Result := '';
end;

function IsTextEncoderSupported: Boolean;
begin
  Result := False;
end;


{ TEncryptionBase }

procedure TEncryptionBase.Decrypt(AEncryptedData: TJSArrayBuffer;
  AResultType: TCryptoDecryptResultType);
begin

end;

procedure TEncryptionBase.Encrypt(APlainText: string);
begin

end;

procedure TEncryptionBase.Encrypt(ABinary: TJSUint8Array);
begin

end;

{ TWebAESEncryption }

constructor TWebAESEncryption.Create(AType: TAESEncryptionType);
begin

end;

constructor TWebAESEncryption.Create(AType: TAESEncryptionType;
  AKeyLength: TAESEncryptionKeyLength);
begin

end;

constructor TWebAESEncryption.Create(AType: TAESEncryptionType;
  AKeyLength: TAESEncryptionKeyLength; AExtractable: Boolean);
begin

end;

constructor TWebAESEncryption.Create(AType: TAESEncryptionType;
  AKeyLength: TAESEncryptionKeyLength; AExtractable: Boolean;
  AUsages: TCryptoKeyUsages);
begin

end;

procedure TWebAESEncryption.Decrypt(AEncryptedData: TJSArrayBuffer;
  AResultType: TCryptoDecryptResultType);
begin
  inherited;

end;

procedure TWebAESEncryption.Encrypt(APlainText: string);
begin

end;

procedure TWebAESEncryption.Encrypt(ABinary: TJSUint8Array);
begin

end;

procedure TWebAESEncryption.ExportKey(AFormat: TCryptoExportImportFormat);
begin

end;

procedure TWebAESEncryption.GenerateKey;
begin

end;

procedure TWebAESEncryption.ImportKey(ABinary: TJSUint8Array);
begin

end;

procedure TWebAESEncryption.ImportKey(ARaw: TJSArrayBuffer);
begin

end;

procedure TWebAESEncryption.ImportKey(AJSON: string);
begin

end;

procedure TWebAESEncryption.UnwrapKey(AImportFormat: TCryptoExportImportFormat;
  AKey: TJSArrayBuffer; AKeyAlgorithm: JSValue; AExtractable: Boolean;
  AKeyUsages: TCryptoKeyUsages);
begin

end;

procedure TWebAESEncryption.WrapKey(AKey: TJSCryptoKey;
  AExportFormat: TCryptoExportImportFormat);
begin

end;

{ TWebRSAEncryption }

constructor TWebRSAEncryption.Create(AModulusLength: TRSAModulusLength;
  AHash: TCryptoHash);
begin

end;

constructor TWebRSAEncryption.Create(AModulusLength: TRSAModulusLength);
begin

end;

constructor TWebRSAEncryption.Create;
begin

end;

constructor TWebRSAEncryption.Create(AModulusLength: TRSAModulusLength;
  AHash: TCryptoHash; AExtractable: Boolean);
begin

end;

constructor TWebRSAEncryption.Create(AModulusLength: TRSAModulusLength;
  AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages);
begin

end;

procedure TWebRSAEncryption.Decrypt(AEncryptedData: TJSArrayBuffer;
  AResultType: TCryptoDecryptResultType);
begin
  inherited;

end;

procedure TWebRSAEncryption.Encrypt(APlainText: string);
begin

end;

procedure TWebRSAEncryption.Encrypt(ABinary: TJSUint8Array);
begin

end;

procedure TWebRSAEncryption.ExportKey(AKeyType: TCryptoAsymKeyType;
  AFormat: TCryptoExportImportFormat);
begin

end;

procedure TWebRSAEncryption.GenerateKey;
begin

end;

procedure TWebRSAEncryption.ImportKey(ABinary: TJSUint8Array;
  AKeyType: TCryptoAsymKeyType);
begin

end;

procedure TWebRSAEncryption.ImportKey(AKey: string;
  AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat);
begin

end;

procedure TWebRSAEncryption.UnwrapKey(AImportFormat: TCryptoExportImportFormat;
  AKey: TJSArrayBuffer; AKeyAlgorithm: JSValue; AExtractable: Boolean;
  AKeyUsages: TCryptoKeyUsages);
begin

end;

procedure TWebRSAEncryption.WrapKey(AKey: TJSCryptoKey;
  AExportFormat: TCryptoExportImportFormat);
begin

end;

{ TSignatureBase }

procedure TSignatureBase.Sign(AText: string);
begin

end;

procedure TSignatureBase.Sign(ABinary: TJSUint8Array);
begin

end;

procedure TSignatureBase.Verify(ASignature, AData: TJSArrayBuffer);
begin

end;

{ TWebRSASignature }

constructor TWebRSASignature.Create;
begin

end;

constructor TWebRSASignature.Create(AModulusLength: TRSAModulusLength);
begin

end;

constructor TWebRSASignature.Create(AModulusLength: TRSAModulusLength;
  AHash: TCryptoHash);
begin

end;

constructor TWebRSASignature.Create(AModulusLength: TRSAModulusLength;
  AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages);
begin

end;

constructor TWebRSASignature.Create(AModulusLength: TRSAModulusLength;
  AHash: TCryptoHash; AExtractable: Boolean);
begin

end;

procedure TWebRSASignature.ExportKey(AKeyType: TCryptoAsymKeyType;
  AFormat: TCryptoExportImportFormat);
begin

end;

procedure TWebRSASignature.GenerateKey;
begin

end;

procedure TWebRSASignature.ImportKey(ABinary: TJSUint8Array;
  AKeyType: TCryptoAsymKeyType);
begin

end;

procedure TWebRSASignature.ImportKey(AKey: string; AKeyType: TCryptoAsymKeyType;
  AFormat: TCryptoExportImportFormat);
begin

end;

procedure TWebRSASignature.Sign(ABinary: TJSUint8Array);
begin

end;

procedure TWebRSASignature.Sign(AText: string);
begin

end;

procedure TWebRSASignature.Verify(ASignature, AData: TJSArrayBuffer);
begin
  inherited;

end;

{ TWebHMACSignature }

constructor TWebHMACSignature.Create;
begin

end;

constructor TWebHMACSignature.Create(AHash: TCryptoHash);
begin

end;

constructor TWebHMACSignature.Create(AHash: TCryptoHash; AExtractable: Boolean;
  AUsages: TCryptoKeyUsages);
begin

end;

constructor TWebHMACSignature.Create(AHash: TCryptoHash; AExtractable: Boolean);
begin

end;

procedure TWebHMACSignature.ExportKey(AFormat: TCryptoExportImportFormat);
begin

end;

procedure TWebHMACSignature.GenerateKey;
begin

end;

procedure TWebHMACSignature.ImportKey(ARaw: TJSArrayBuffer);
begin

end;

procedure TWebHMACSignature.ImportKey(ABinary: TJSUint8Array);
begin

end;

procedure TWebHMACSignature.ImportKey(AJSON: string);
begin

end;

procedure TWebHMACSignature.Sign(ABinary: TJSUint8Array);
begin

end;

procedure TWebHMACSignature.Sign(AText: string);
begin

end;

procedure TWebHMACSignature.Verify(ASignature, AData: TJSArrayBuffer);
begin
  inherited;

end;

end.
