{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Devices;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, SysUtils, WEBLib.Controls, Web, JS,
  WEBLib.ExtCtrls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, WEBLib.Controls, Web, JS,
  WEBLib.ExtCtrls;
  {$ENDIF}

type
  TCameraType = (ctFront, ctRear, ctSelected);
  TCameraBaseFormat = (cbfJPEG, cbfPNG);

  TCameraDevice = class(TCollectionItem)
  private
    FId: string;
    FName: string;
  public
    procedure Assign(Source: TPersistent); override;
    property Id: string read FId write FId;
    property Name: string read FName write FName;
  end;

  TCameraDevices = class(TCollection)
  private
    procedure SetItem(Index: Integer; const Value: TCameraDevice);
    function GetItem(Index: Integer): TCameraDevice;
  public
    constructor Create;
    function Add: TCameraDevice;
    function GetDeviceById(Id: string): TCameraDevice;
    function GetDeviceByName(Name: string): TCameraDevice;
    function Insert(Index: Integer): TCameraDevice;
    property Items[Index: integer]: TCameraDevice read GetItem write SetItem;
  end;

  TCameraEvent = procedure(Sender: TObject; ACamera: TCameraDevice) of object;
  TCameraBeforeStartEvent = procedure(Sender: TObject; var AConstraints: TJSObjectRecord) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebCamera = class(TWebCustomControl)
  private
    FOnCameraStop: TCameraEvent;
    FSnapShotAsImageData: TJSImageData;
    FSnapShotAsBase64: string;
    FOnSwitchCamera: TCameraEvent;
    FCameraDevices: TCameraDevices;
    FOnCameraStreamPlay: TCameraEvent;
    FPaused: Boolean;
    FOnCameraPause: TCameraEvent;
    FOnCameraDevicesInitialized: TNotifyEvent;
    FOnCameraResume: TCameraEvent;
    FCameraType: TCameraType;
    FCameraBaseFormat: TCameraBaseFormat;
    FSnapShotAsUint8Array: TJSUint8Array;
    FOnBeforeStart: TCameraBeforeStartEvent;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function BrowserSupportedConstraints: TStringList;
    procedure Pause;
    procedure Resume;
    procedure SetSelectedCameraDevice(aDevice: TCameraDevice);
    procedure Start;
    procedure Stop;
    property CameraDevices: TCameraDevices read FCameraDevices;
    property Paused: Boolean read FPaused;
    property SnapShotAsBase64: string read FSnapShotAsBase64;
    property SnapShotAsImageData: TJSImageData read FSnapShotAsImageData;
    property SnapShotAsUint8Array: TJSUint8Array read FSnapShotAsUint8Array;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BaseFormat: TCameraBaseFormat read FCameraBaseFormat write FCameraBaseFormat default cbfJPEG;
    property CameraType: TCameraType read FCameraType write FCameraType default ctFront;
    property Visible;
    property OnBeforeStart: TCameraBeforeStartEvent read FOnBeforeStart write FOnBeforeStart;
    property OnCameraDevicesInitialized: TNotifyEvent read FOnCameraDevicesInitialized write FOnCameraDevicesInitialized;
    property OnCameraPause: TCameraEvent read FOnCameraPause write FOnCameraPause;
    property OnCameraResume: TCameraEvent read FOnCameraResume write FOnCameraResume;
    property OnCameraStreamPlay: TCameraEvent read FOnCameraStreamPlay write FOnCameraStreamPlay;
    property OnCameraStop: TCameraEvent read FOnCameraStop write FOnCameraStop;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnSwitchCamera: TCameraEvent read FOnSwitchCamera write FOnSwitchCamera;
  end;

  TMediaCaptureType = (mctVideo, mctAudio, mctBoth);
  TMediaCaptureRecordingMode = (mrmManual, mrmAuto);
  TMediaCaptureFFTSize = (mfs32, mfs64, mfs128, mfs256, mfs512, mfs1024, mfs2048, mfs4096, mfs8192, mfs16384, mfs32768);
  TMediaCaptureCloseEvent = procedure(Sender: TObject; ABinary: TJSUint8Array; ABase: string) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebMediaCapture = class(TComponent)
  private
    FOnStopCapture: TMediaCaptureCloseEvent;
    FOnStartCapture: TNotifyEvent;
    FCamera: TWebCamera;
    FCapture: TMediaCaptureType;
    FFFTSize: TMediaCaptureFFTSize;
    FRecordingMode: TMediaCaptureRecordingMode;
    FSensitivity: Integer;
    FTimeConstant: Double;
    function IsTimeConstantStored: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Start;
    procedure Stop;
  published
    property Capture: TMediaCaptureType read FCapture write FCapture default mctBoth;
    property Camera: TWebCamera read FCamera write FCamera;
    property FFTSize: TMediaCaptureFFTSize read FFFTSize write FFFTSize default mfs2048;
    property RecordingMode: TMediaCaptureRecordingMode read FRecordingMode write FRecordingMode default mrmManual;
    property Sensitivity: Integer read FSensitivity write FSensitivity default 100;
    property SmoothingTimeConstant: Double read FTimeConstant write FTimeConstant stored IsTimeConstantStored;
    property OnStartCapture: TNotifyEvent read FOnStartCapture write FOnStartCapture;
    property OnStopCapture: TMediaCaptureCloseEvent read FOnStopCapture write FOnStopCapture;
  end;

  TDecoderDecodedEvent = procedure(Sender: TObject; ADecoded: string) of object;

  TDecoder = class(TComponent)
  private
    FCamera: TWebCamera;
    FOnDecoded: TDecoderDecodedEvent;
    FEnableTimer: Boolean;
  protected
    procedure DecodeURL(AURL: string); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    property Camera: TWebCamera read FCamera write FCamera;
    property EnableTimer: Boolean read FEnableTimer write FEnableTimer default False;
    property OnDecoded: TDecoderDecodedEvent read FOnDecoded write FOnDecoded;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebQRDecoder = class(TDecoder)
  public
    procedure DecodeURL(AURL: string); override;
  published
    property Camera;
    property EnableTimer;
    property OnDecoded;
  end;

  TBarcodeDecoderReader = (bcvCode128, bcvEAN, bcvEAN8, bcvCode39, bcvCode39VIN, bcvCodabar, bcvUPC, bcvUPCE, bcvI2OF5, bcv2OF5, bcvCode93);
  TBarcodeDecoderReaders = set of TBarcodeDecoderReader;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebBarcodeDecoder = class(TDecoder)
  private
    FReader: TBarcodeDecoderReaders;
  public
    constructor Create(AOwner: TComponent); override;
    procedure DecodeURL(AURL: string); override;
  published
    property Camera;
    property EnableTimer;
    property Reader: TBarcodeDecoderReaders read FReader write FReader default [bcvCode128];
    property OnDecoded;
  end;

implementation

{ TCameraDevice }

procedure TCameraDevice.Assign(Source: TPersistent);
begin
  if (Source is TCameraDevice) then
  begin
    FName := (Source as TCameraDevice).Name;
    FId := (Source as TCameraDevice).Id;
  end
  else
    inherited;
end;

{ TCameraDevices }

function TCameraDevices.Add: TCameraDevice;
begin
  Result := TCameraDevice(inherited Add);
end;

constructor TCameraDevices.Create;
begin
  inherited Create(TCameraDevice);
end;

function TCameraDevices.GetDeviceById(Id: string): TCameraDevice;
begin
  Result := nil;
end;

function TCameraDevices.GetDeviceByName(Name: string): TCameraDevice;
begin
  Result := nil;
end;

function TCameraDevices.GetItem(Index: Integer): TCameraDevice;
begin
  Result := TCameraDevice(inherited Items[Index]);
end;

function TCameraDevices.Insert(Index: Integer): TCameraDevice;
begin
  Result := TCameraDevice(inherited Insert(Index));
end;

procedure TCameraDevices.SetItem(Index: Integer; const Value: TCameraDevice);
begin
  inherited Items[Index] := Value;
end;

{ TWebCamera }

function TWebCamera.BrowserSupportedConstraints: TStringList;
begin
  Result := nil;
end;

constructor TWebCamera.Create(AOwner: TComponent);
begin
  inherited;
  Width := 400;
  Height := 300;
  FCameraDevices := TCameraDevices.Create;
end;

destructor TWebCamera.Destroy;
begin
  FCameraDevices.Free;
  inherited;
end;

procedure TWebCamera.Pause;
begin
  //
end;

procedure TWebCamera.Resume;
begin
  //
end;

procedure TWebCamera.SetSelectedCameraDevice(aDevice: TCameraDevice);
begin
  //
end;

procedure TWebCamera.Start;
begin
  //
end;

procedure TWebCamera.Stop;
begin
  //
end;

{ TDecoder }

constructor TDecoder.Create(AOwner: TComponent);
begin
  inherited;
  FEnableTimer := False;
end;

procedure TDecoder.DecodeURL(AURL: string);
begin
  //
end;

destructor TDecoder.Destroy;
begin
  //
  inherited;
end;

procedure TDecoder.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if not (csDestroying in ComponentState) then
  begin
    if (Operation = opRemove) then
    begin
      if (AComponent = FCamera) then
        FCamera := nil;
    end;
  end;
end;

{ TWebQRDecoder }

procedure TWebQRDecoder.DecodeURL(AURL: string);
begin
  inherited;
end;

{ TWebBarcodeDecoder }

constructor TWebBarcodeDecoder.Create(AOwner: TComponent);
begin
  inherited;
  FReader := [bcvCode128];
end;

procedure TWebBarcodeDecoder.DecodeURL(AURL: string);
begin
  inherited;
end;

{ TWebMediaCapture }

constructor TWebMediaCapture.Create(AOwner: TComponent);
begin
  inherited;
  FCapture := mctBoth;
  FFFTSize := mfs2048;
  FSensitivity := 100;
  FTimeConstant := 0.9;
  FRecordingMode := mrmManual;
end;

function TWebMediaCapture.IsTimeConstantStored: Boolean;
begin
  Result := FTimeConstant <> 0.9;
end;

procedure TWebMediaCapture.Start;
begin
  //
end;

procedure TWebMediaCapture.Stop;
begin
  //
end;

end.
